var sqlBuilder = require('json-sql');
//var mongo = require('mongodb');
const pg = require('pg');
const uuidV4 = require('uuid/v4');

/**
 * @author Max Tushev <maximtushev@gmail.com>
 * @class Database.drivers.Oracle.Database
 * @extend: Database.drivers.Mysql.Database
 * The connector to Oracle database
 */
Ext.define("Database.drivers.Postgresql.Database", {

    extend: 'Database.drivers.Mysql.Database'

    ,constructor: function(cfg) {
        var me = this;
        this.cfg = cfg;        
        this.debugQuery = !!cfg.debugQuery;
        this.debug = !!cfg.debug;        
         
        //this.callParent()
        this.readFieldTypes(function() {
            me.connect(cfg, cfg.callback)
        })
        
    }    
    
    ,connect: function(cfg, callback) {        
        if(!cfg) return;        
        let client = new pg.Client(cfg);        
        
        this.ID = uuidV4();
        
        client.on('error', (err, client) => {            
            if(!!cfg.onError) {
                cfg.onError(err, client)
            }            
            //console.error('PG error', err.message, err.stack);
        });
        
        client.connect((e, c) => {
            if(e) {
               console.log(e)
               process.exit();
               return;
            }            
            this.conn = client;
            
            if(cfg.schema) {
                this.query("SET SCHEMA '" + cfg.schema + "'",() => {
                    if(!!callback)
                        callback(client); 
                })
            } else            
            if(!!callback)
                callback(client);            

        })
    }

    ,getCollections: function(callback) {
        this.conn.query("SELECT table_name FROM information_schema.tables WHERE table_schema='"+(this.cfg.schema || 'public')+"'   AND table_type='BASE TABLE'", [], (err, res) => {

            if(res) { 
                res.rows.each((item) => {
                    return item.table_name;
                },true)            
                callback(res.rows)
            } else callback([])
        })
    }
    
    ,getPointedParams: function(o) {
        var out = [];
        var f = function(o) {
            if(Ext.isObject(o)) {
                for(var i in o) {
                    if(/\./.test(i) && out.indexOf(i) == -1) {
                        out.push(i)
                    }
                    if(Ext.isObject(o[i]) || Ext.isArray(o[i])) {
                        f(o[i])
                    }
                }
            } else
            if(Ext.isArray(o)) {
                o.forEach(function(o) {
                    f(o)
                })
            }        
        }
        f(o)
        return out;
    }
    
    ,queryBuild: function(cfg) {
        if(cfg.fields && Ext.isObject(cfg.fields)) {
            var qFields = []
            for(var i in cfg.fields) if(cfg.fields[i] == 1) qFields.push(i)
            cfg.fields = qFields
        }
        
        var offset = 0, limit;
        
        if(cfg.limit) {
            limit = cfg.limit;
            delete cfg.limit;
        }
        if(cfg.offset || cfg.offset === 0) {
            offset = cfg.offset;
            delete cfg.offset;
        }
        var pointedParams = this.getPointedParams(cfg.condition)

        var res = sqlBuilder(cfg);
        //Search with ilike
        if(res.query.indexOf('like')>=0){
            res.query=res.query.replace(new RegExp('like', 'g'), 'ilike');
        }
        if(pointedParams && pointedParams.length) {
            pointedParams.forEach(function(p) {
                var x = p.split('.')
                res.query = res.query.replace(new RegExp(x[0]+'\.'+x[1], 'g'), x[0] + "->>'" +x[1]+ "'")
            })
        }        
        out = []
        for(var i in res.values) {
            if(Ext.isBoolean(res.values[i]))
                out.push(res.values[i]? 1:0)
            else
                out.push(res.values[i])    
        }
        var p = 1, i = 0;
        if(res.query.substr(0,6) == 'select') {
            res.query = res.query.replace(/(.{2}\$p[0-9]{1,})/g, function(str) { 
                    
                var opt = str.substr(0,2);
                
                if(out[i] === null && opt == '= ') {
                    out.splice(i,1);
                    return ' IS NULL';                
                }
                i++;
                return opt + '$' + p++;
            });
        } else {
        
            res.query = res.query.replace(/(\$p[0-9]{1,})/g, function(str) { return '$' + (parseInt(str.replace('$p','')) + 1)});
        }

        //Vaibhav Vaidya, 25 May 2018, Add fix for not equal to condition(one line added).
        res.query = res.query.replace(/! IS NULL/g, 'IS NOT NULL');

        res.query = res.query.replace(/___ =/g, ' @>').replace(/\slike\s/g, ' ilike ')

        
        //res.query = this.prepFieldsNamesInQuery(res.query);
        
        res.values = out
        res.query = res.query.slice(0,-1)

        if(limit) {
            res.query += ' LIMIT ' + limit + (offset? ' OFFSET ' + offset:'');
        }
 
        return res
    }
    
                
    ,createCollection: function(collection, callback) {
        
        if(this.notCheckCollection) {
            callback();
            return;
        }
        
        var me = this;
        [
            function(call) {               
                var sql = [
                    'CREATE table '+collection+' (',
                    '_id        varchar(36) NOT NULL DEFAULT uuid_generate_v4(),',
                    'mtime      timestamp,',
                    'ctime      timestamp,',
                    'ltime      timestamp,',
                    'stime      bigint,',
                    'removed    int  NOT NULL DEFAULT 0,',
                    'signobject jsonb,',
                    'maker      varchar(36))'
                ].join('');
                me.conn.query(sql, [], function(e, d) {
                    if(e) {
                        console.log('create error:', sql)
                        //me.error(e)
                        callback()
                    } else
                        callback();
                })
            }
            
        ].runEach()
    }
    


    ,getData: function(collection, find, fields, sort, start, limit, callback) {
        var me = this
            ,fieldsAr = []
            ,cfg = {
                type: 'select',
                table: collection,
                condition: find                
            };
            
        [
            // get counts
            function(call) {
                cfg.fields = ["count(*) as cnt"]
                var res = me.queryBuild(cfg)
                me.query(res.query, res.values, function(e, data) {
                    if(data && data[0] && data[0].cnt) 
                        call(data[0].cnt)    
                    else
                        callback(0, [])
                })
            }
            
            ,function(total) {
                cfg.fields = fields
                if(sort) cfg.sort = sort
                if(limit) cfg.limit = limit
                if(start || start === 0) cfg.offset = start

                me.collection(cfg.collection).query(cfg, function(err, res) {
                    if(me.debug && err) {
                        console.log('Database getData err:', err)
                        console.log('getData res:', cfg)     
                    }
                    callback(total, res)
                })

            }
        ].runEach();
    }

    
    ,async query(query, arg1, arg2) {
        let values, callback;

        if(Ext.isFunction(arg1)) {
            callback = arg1;
            values = [];
        } else {
            callback = arg2;
            values = arg1;
        }

        try {
            const { rows } = await this.conn.query(query, values);            
            rows.forEach((r,i) => {
                for(var j in r) {            
                    if(r[j] && !!r[j]._arr && Object.keys(r[j]).length == 1)
                        rows[i][j] = r[j]._arr;                
                }
            })
            if(!!callback)
                callback(null, rows);
            else
                return rows;        
        } catch(e) {
            console.log('PG ERROR:', query, e);
            console.trace("Here I am!")
            if(!!callback)
                callback(e, null);
            else
                return null;
        }
    }

    ,createObjectId: function(a1, a2) {
        let cb;
        
        if(Ext.isFunction(a1)) cb = a1;
        else
        if(Ext.isFunction(a2)) cb = a2;
        if(!!cb)
            cb(uuidV4());//new mongo.BSONPure.ObjectID())
        else
            return uuidV4();//new mongo.BSONPure.ObjectID();
    }
    
    ,begin: function(cb) {
        var me = this;
        this.inTransaction = true;
        this.conn.query('COMMIT', function (err) {
            me.conn.query('BEGIN', function (err) {
                if (err) {
                    cb(err, false)
                } else {
                    
                    cb(null, true)
                }
            })
        })
    }
    ,commit: function(cb) {
        this.inTransaction = false;
        var me = this;
        this.conn.query('COMMIT', function (err) {
            if (err) {
                cb(err, false)
            } else {
                me.isBusy = false;
                me.startTime = 0;
                cb(null, true)
            }
            
        });
    }
    ,rollback: function(cb) {
        var me = this;
        this.inTransaction = false;
        this.conn.query('ROLLBACK', function (err) {
            if (err) {
                cb(err, false)
            } else {
                me.isBusy = false;
                me.startTime = 0;
                cb(err, true)
            }
            
        });
    }
    
    ,checkIndex: function(field, table, cb) {
        if(this.inTransaction) {
            cb();
            return;            
        };
        [
            function(next) {
                this.conn.query("SELECT  * from pg_indexes where tablename = '"+table+"' and indexname = '"+table+"_"+field.name+"'", function(e,d) { 

                    if(d.rows.lngth) {
                        cb();
                    } else {
                        next();
                    }
                })
            }
            ,function() {
                var sql;
                if(this.fieldTypes[field.type] && !!this.fieldTypes[field.type].createIndex) {
                    sql = this.fieldTypes[field.type].createIndex(field.name, table);
                } else {
                    sql = "CREATE INDEX " +table+ "_"+field.name+" ON " +table+ " ("+field.name+")";
                }            
                this.conn.query(sql, function(e) {
                    cb()})
            }
        ].runEach(this);
    }

    
})
